/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#include "app_context.h"
#include <coemain.h>
#include <flogger.h>
#include <bautils.h>


class CDiskSpaceNotifier: public CCheckedActive
{
public:
	static CDiskSpaceNotifier* NewL(RFs& fs, TInt aDrive, MDiskSpace* aNotifier, TInt64 aThreshold);
	~CDiskSpaceNotifier();
private:
	CDiskSpaceNotifier(RFs& fs, TInt aDrive, MDiskSpace* aNotifier, TInt64 aThreshold);
	void ConstructL();
	void CheckedRunL();
	void DoCancel();

	MDiskSpace* iNotifier;
	RFs& iFs;
	TInt iDrive;
	TInt64 iThreshold;
};

EXPORT_C void print_stacktrace(RFs& fs, MApp_context* context, const TDesC& cat, TInt error)
{
	if (!context) return;
	RFile f;
	if (f.Open(fs, _L("C:\\crash.txt"), 
		EFileShareAny | EFileStreamText | EFileWrite)!=KErrNone) {
		if(f.Create(fs, _L("C:\\crash.txt"), 
			EFileShareAny | EFileStreamText | EFileWrite)!=KErrNone) {
			return;
		}
	}
	CleanupClosePushL(f);

	TInt pos=0;
	f.Seek(ESeekEnd, pos);

	TBuf<80> msg;
	msg.Format(_L("context_log crashed with error %d\ncategory "), error);
	TPtrC8 msg2( (TText8*)msg.Ptr(), msg.Length()*2);
	f.Write(msg2);
	TPtrC8 msg3( (TText8*)cat.Ptr(), cat.Length()*2);
	f.Write(msg3);
	f.Write(_L8("\ntrace:\n"));

	TPtr n=context->PopCallStack();
	while (n.Ptr()) {
		TPtrC8 msg2( (TText8*)n.Ptr(), n.Length()*2);
		f.Write(msg2);
		n=context->PopCallStack();
	}
	CleanupStack::PopAndDestroy();
}

EXPORT_C void exceptionhandler(TExcType t)
{
	MApp_context* c;
	c=MApp_context::Static();
	if (c) {
		print_stacktrace(c->Fs(), c, _L("Exception"), t);
	}
	User::Panic(_L("EXCEPTION"), t);
}

MApp_context::~MApp_context()
{
}

CApp_context::CApp_context()
{
}

void CApp_context::ConstructL(bool aFsOnly, const TDesC& Name)
{
	iFsOnly=aFsOnly;

	iDiskSpaceLimit=500*1024; // 500k
	iDrive[0]=2; // C:
	iFileLog=0;

	User::LeaveIfError(iFs.Connect());
	iSpaceLeft=true;

	if (iCallStackChunk.OpenGlobal(Name, EFalse)==KErrNone) {
		has_chunk=true;
		iCallStackZero=iCallStackCurrent=(char*)iCallStackChunk.Base() + 4;
		iCallStackSize=iCallStackChunk.Size();
	} else {
		has_chunk=false;
		iCallStackZero=new char[4096];
		if (iCallStackZero) iCallStackZero += 4;
		iCallStackCurrent=iCallStackZero;
		iCallStackSize=4096;
	}

	if (!aFsOnly) {
		iDiskSpaceNotifier[0]=CDiskSpaceNotifier::NewL(iFs, 2, this, iDiskSpaceLimit);


#ifndef __WINS__
		{
		iDrive[1]=-1;
		TDriveInfo i;
		if (iFs.Drive(i, EDriveE)==KErrNone) {
			if (i.iType!=EMediaNotPresent &&
				i.iType!=EMediaUnknown &&
				i.iType!=EMediaCdRom &&
				i.iType!=EMediaRom) {
				// memory card
				iDrive[1]=4;
				iDiskSpaceNotifier[1]=CDiskSpaceNotifier::NewL(iFs, 4, this, iDiskSpaceLimit);
			}
		}
		}
#endif

		int v;
		for(v=0; v<2; v++) {
			if (iDrive[v] < 0) continue;
			TVolumeInfo volinfo;
			if (iFs.Volume(volinfo, iDrive[v])==KErrNone) {
				if (volinfo.iFree<=iDiskSpaceLimit) {
					iSpaceLeft=false;
				}
			}
		}

		globalNote=CAknGlobalNote::NewL();

		User::LeaveIfError(iSysAgent.Connect());

#ifndef __WINS__
		User::LeaveIfError(iTelServer.Connect());
		_LIT(KGsmModuleName, "phonetsy.tsy");
		User::LeaveIfError( iTelServer.LoadPhoneModule( KGsmModuleName ) );
		RTelServer::TPhoneInfo info;
		User::LeaveIfError( iTelServer.GetPhoneInfo( 0, info ) );
		User::LeaveIfError( iPhone.Open( iTelServer, info.iName ) );
#endif
	}

	cc=CCnvCharacterSetConverter::NewL();
	cc->PrepareToConvertToOrFromL(KCharacterSetIdentifierIso88591, iFs);

	Dll::SetTls(this);

	RFileLogger iLog;
	iLog.Connect();
	iLog.CreateLog(_L("ContextCommon"),_L("CApp_context"),EFileLoggingModeAppend);
	iLog.Write(_L("constructed"));

	iLog.CloseLog();
	iLog.Close();

}

CApp_context::~CApp_context()
{
	Dll::SetTls(0);

	delete iDiskSpaceNotifier[0];
	delete iDiskSpaceNotifier[1];

	delete iSettings;
	delete globalNote;

	if (!has_chunk) {
		iCallStackZero-=4;
		delete iCallStackZero;
	} else {
		iCallStackZero-=4;
		*(TInt*)iCallStackZero=0;
		iCallStackChunk.Close();
	}
	iCallStackZero=iCallStackCurrent=0;

	if (!iFsOnly) {
#ifndef __WINS__
		iPhone.Close();
		_LIT(KGsmModuleName, "phonetsy.tsy");
		iTelServer.UnloadPhoneModule( KGsmModuleName );
		iTelServer.Close();
#endif
		iSysAgent.Close();
	}

	delete cc;
	iFs.Close();

	RFileLogger iLog;
	iLog.Connect();
	iLog.CreateLog(_L("ContextCommon"),_L("CApp_context"),EFileLoggingModeAppend);
	iLog.Write(_L("deleted"));

	iLog.CloseLog();
	iLog.Close();
}

EXPORT_C CApp_context* CApp_context::NewL(bool aFsOnly, const TDesC& Name)
{
	CApp_context* ret;
	ret=new (ELeave) CApp_context();
	CleanupStack::PushL(ret);
	ret->ConstructL(aFsOnly, Name);
	CleanupStack::Pop();
	return ret;
}

RFs& CApp_context::Fs()
{
	return iFs;
}

RSystemAgent& CApp_context::SysAgent()
{
	return iSysAgent;
}

RTelServer& CApp_context::TelServer()
{
	return iTelServer;
}

RAdvGsmPhone& CApp_context::Phone()
{
	return iPhone;
}

CCnvCharacterSetConverter* CApp_context::CC()
{
	return cc;
}

void CApp_context::SetFileLog(MPausable* log)
{
	iFileLog=log;
	if (iFileLog && !iSpaceLeft) iFileLog->pause();
}

void CApp_context::DiskSpaceThreshold(TInt aDrive)
{
	int v;
	iSpaceLeft=true;
	TBuf<100> msg;
	TAknGlobalNoteType notetype;
	for(v=0; v<2; v++) {
		if (iDrive[v] < 0) continue;

		TVolumeInfo info;
		if (iFs.Volume(info, iDrive[v])==KErrNone) {
			if (info.iFree<=iDiskSpaceLimit) {
				iSpaceLeft=false;
			}
		}
	}
	if (!iSpaceLeft) {
		if (iFileLog) iFileLog->pause();
		msg.Append(_L("WARNING! No space left for logs. Please delete some pictures and other media."));
		notetype=EAknGlobalErrorNote;
	} else {
		if (iFileLog) iFileLog->unpause();
		msg.Append(_L("Logging continues."));	
		notetype=EAknGlobalInformationNote;
	}
	if (globalNote) {
		TRAPD(err, globalNote->ShowNoteL(notetype, msg));
	}
}

void CDiskSpaceNotifier::ConstructL()
{
	CActiveScheduler::Add(this);
	iFs.NotifyDiskSpace(iThreshold, iDrive, iStatus);
	SetActive();
}

CDiskSpaceNotifier* CDiskSpaceNotifier::NewL(RFs& fs, TInt aDrive, MDiskSpace* aNotifier, TInt64 aThreshold)
{
	auto_ptr<CDiskSpaceNotifier> ret(new (ELeave) CDiskSpaceNotifier(fs, aDrive, aNotifier, aThreshold));
	ret->ConstructL();
	return ret.release();
}

CDiskSpaceNotifier::~CDiskSpaceNotifier()
{
	Cancel();
}

CDiskSpaceNotifier::CDiskSpaceNotifier(RFs& fs, TInt aDrive, MDiskSpace* aNotifier, TInt64 aThreshold) : 
	CCheckedActive(EPriorityNormal, _L("CDiskSpaceNotifier")), iFs(fs), iDrive(aDrive),
	iNotifier(aNotifier), iThreshold(aThreshold)
{
}

void CDiskSpaceNotifier::CheckedRunL()
{
	if (iStatus==KErrNone) {
		iNotifier->DiskSpaceThreshold(iDrive);
	}
	iFs.NotifyDiskSpace(iThreshold, iDrive, iStatus);
	SetActive();
}

void CDiskSpaceNotifier::DoCancel()
{
	iFs.NotifyDiskSpaceCancel();
}

bool CApp_context::NoSpaceLeft()
{
	return !iSpaceLeft;
}

const TDesC& CApp_context::DataDir()
{
	return iDataDir;
}

void CApp_context::SetDataDir(const TDesC& Dir)
{
	BaflUtils::EnsurePathExistsL(iFs, Dir);
	TDriveInfo i;
	if (iFs.Drive(i, EDriveE)==KErrNone) {
		if (i.iType!=EMediaNotPresent &&
			i.iType!=EMediaUnknown &&
			i.iType!=EMediaCdRom &&
			i.iType!=EMediaRom) {

			// memory card

			TFileName f=Dir;
			f.Replace(0, 1, _L("E"));
			TRAPD(err, BaflUtils::EnsurePathExistsL(iFs, f));
		}
	}
	iDataDir=Dir;
}

const TDesC& CApp_context::AppDir()
{
	return iAppDir;
}

void CApp_context::SetAppDir(const TDesC& Dir)
{
	iAppDir=Dir;
}

void CApp_context::SetSettings(MSettings* Settings)
{
	iSettings=Settings;
}

MSettings& CApp_context::Settings()
{
	return *iSettings;
}

RWsSession& CApp_context::Ws()
{
	return CCoeEnv::Static()->WsSession();
}

const char* CApp_context::CallStack()
{
	return iCallStackCurrent;
}

void CApp_context::SetActiveErrorReporter(MActiveErrorReporter* Reporter)
{
	iReporter=Reporter;
}

void CApp_context::ReportActiveError(const TDesC& Source,
	const TDesC& Reason, TInt Code)
{
	if (iReporter) {
		iReporter->ReportError(Source, Reason, Code);
	} else if (globalNote) {
		TBuf<256> msg;
		msg.Format(_L("Unhandled error %d %S %S"), Code, &(Source.Left(100)), &(Reason.Left(120)));
		globalNote->ShowNoteL(EAknGlobalErrorNote, msg);
	}
}

EXPORT_C MApp_context* MApp_context::Static()
{
	return (MApp_context*)Dll::Tls();
}

EXPORT_C const TDesC& MContextBase::DataDir()
{
	return iContext.DataDir();
}

EXPORT_C const TDesC& MContextBase::AppDir()
{
	return iContext.AppDir();
}

EXPORT_C MSettings& MContextBase::Settings()
{
	return iContext.Settings();
}


EXPORT_C bool MContextBase::NoSpaceLeft()
{
	return iContext.NoSpaceLeft();
}

EXPORT_C MContextBase::MContextBase() : iContext(*(MApp_context::Static()))
{
}

EXPORT_C MContextBase::MContextBase(MApp_context& Context) : iContext(Context)
{
}


EXPORT_C MContextBase::~MContextBase()
{
}

EXPORT_C RFs& MContextBase::Fs()
{
	return iContext.Fs();
}

EXPORT_C RSystemAgent& MContextBase::SysAgent()
{
	return iContext.SysAgent();
}

EXPORT_C RTelServer& MContextBase::TelServer()
{
	return iContext.TelServer();
}

EXPORT_C RAdvGsmPhone& MContextBase::Phone()
{
	return iContext.Phone();
}

EXPORT_C MApp_context& MContextBase::AppContext()
{
	return iContext;
}

EXPORT_C CCnvCharacterSetConverter* MContextBase::CC()
{
	return iContext.CC();
}

EXPORT_C RWsSession& MContextBase::Ws()
{
	return iContext.Ws();
}

EXPORT_C void MContextBase::ReportActiveError(const TDesC& Source,
	const TDesC& Reason, TInt Code)
{
	iContext.ReportActiveError(Source, Reason, Code);
}


EXPORT_C MApp_context* GetContext()
{
	return MApp_context::Static();
}

EXPORT_C MApp_context* MContextBase::GetContext() const
{
	return &iContext;
}

TInt CApp_context::PushCallStack(const TDesC& Name)
{
	if (!iCallStackZero) return KErrNotFound;
	if (iCallStackCurrent+Name.Length()*2+8 > iCallStackCurrent+iCallStackSize) return KErrNoMemory;

	TInt len, added;
	len=Name.Length()*2;
	// align on 4 byte boundaries
	added=(4-(len % 4));
	if (added==4) added=0;
	len+=added;

	TPtr pt( (TUint16*)iCallStackCurrent, len/2);
	pt=Name;

	if(added>=2) pt.Append(_L(" "));

	TInt* pp=(TInt*) (iCallStackCurrent+len);
#ifdef __WINS__
	TBuf<200> msg;
	msg.Format(_L("PushCallStack name at %d length %d link at %d next at %d: %S"),
		(TInt)iCallStackCurrent, TInt(len), 
		(TInt)pp, (TInt)(iCallStackCurrent+len + 4), &Name);
	RDebug::Print(msg);
#endif

	*pp=iCallStackCurrent-iCallStackZero;
	iCallStackCurrent+=len + 4;

	*((TInt*) (iCallStackZero-4)) = (iCallStackCurrent - iCallStackZero);

	return KErrNone;
}

TPtr CApp_context::PopCallStack()
{
	if (iCallStackCurrent==iCallStackZero) {
		return TPtr(0, 0);
	}
	TInt* pd=(TInt*)(iCallStackCurrent - 4);
	char* p= iCallStackZero + *pd;
	TInt len=(iCallStackCurrent - p - 4)/2;
	TPtr ret( (TUint16*)(p), len, len);

#ifdef __WINS__
	TBuf<200> msg;
	msg.Format(_L("PopCallStack name at %d length %d link at %d next at %d: %S"),
		(TInt)p, TInt(len*2), 
		(TInt)pd, (TInt)(iCallStackCurrent), &ret);
	RDebug::Print(msg);
#endif

	iCallStackCurrent=p;
	*((TInt*) (iCallStackZero-4)) = (iCallStackCurrent - iCallStackZero);
	return ret;
}

void CApp_context::ResetCallStack()
{
	iCallStackCurrent=iCallStackZero;
	*((TInt*) (iCallStackZero-4)) = 0;
}

class TFormattedStack : public MCallStackVisitor {
public:
	TFormattedStack(HBufC* Into, const TDesC& Prefix) : iInto(Into), iPrefix(Prefix) { }
private:
	virtual void BeginStack()
	{
		iInto->Des().Zero();
		iInto->Des().Append(iPrefix);
		iInto->Des().Append(_L(":"));
	}
	virtual void VisitItem(const TDesC& Name)
	{
		iInto->Des().Append(Name);
		iInto->Des().Append(_L("|"));
	}
	virtual void EndStack()
	{
	}

	HBufC* iInto;
	const TDesC& iPrefix;
};

HBufC* CApp_context::GetFormattedCallStack(const TDesC& prefix)
{
	if (iCallStackCurrent==iCallStackZero) return 0;
	HBufC* ret=0;
	TRAPD(err, ret=HBufC::NewL( (iCallStackCurrent-iCallStackZero)/2) + prefix.Length() + 1);
	if (err!=KErrNone) return 0;

	TFormattedStack f(ret, prefix);
	IterateStack(f);

	return ret;
}


void CApp_context::IterateStack(MCallStackVisitor& aVisitor, const char* aStack)
{
	if (aStack==0) aStack=iCallStackZero-4;

	aVisitor.BeginStack();
	const char* current=aStack+4+*(TInt*)aStack;
	aStack+=4;
	while (current!=aStack) {
		const char *prev=current -4;
		TInt* p=(TInt*)(prev);
		current=aStack + *p;
		TInt len=(prev-current)/2;
		TPtrC n( (TUint16*)current, len);
		aVisitor.VisitItem(n);
	}
	aVisitor.EndStack();
}

EXPORT_C TCallStackItem::TCallStackItem(const TDesC& Name, MApp_context* Context)
{
	iAppContext=Context;

	if (!iAppContext) return;
	TInt err=iAppContext->PushCallStack(Name.Left(100));
	if (err!=KErrNone) {
		iAppContext=0;
	}
}

EXPORT_C TCallStackItem::~TCallStackItem()
{
	if (!iAppContext) return;
	iAppContext->PopCallStack();
}

EXPORT_C void NoAppContext()
{
	Dll::SetTls(0);
}

EXPORT_C void MoveIntoDataDirL(MApp_context_access& Context, const TDesC& FileName)
{
	TFileName appfile, datafile;

	appfile.Format(_L("%S%S"), &Context.AppDir(), &FileName);
	datafile.Format(_L("%S%S"), &Context.DataDir(), &FileName);
	if (BaflUtils::FileExists(Context.Fs(), appfile)) {
		User::LeaveIfError(BaflUtils::CopyFile(Context.Fs(), appfile, datafile));
		User::LeaveIfError(BaflUtils::DeleteFile(Context.Fs(), appfile));
	}
}